@@ -120,7 +120,7 @@ class KindlePW(Kindle):
     output_profile = 'kindle_pw'
 
 class KindleVoyage(Kindle):
-    name = 'Kindle Voyage'
+    name = 'Kindle Voyage/Oasis'
     id = 'kindle_voyage'
     output_profile = 'kindle_voyage'
 
@@ -232,6 +232,11 @@ class CybookOdyssey(CybookOpus):
     name = 'Cybook Odyssey'
     id = 'cybook_odyssey'
 
+class CybookMuse(CybookOpus):
+
+    name = 'Cybook Muse'
+    id = 'cybook_muse'
+    output_profile = 'tablet'
 
 class PocketBook360(CybookOpus):
 
@@ -477,7 +482,7 @@ class StanzaPage(QWizardPage, StanzaUI):
         self.setupUi(self)
         try:
             self.instructions.setText(self.instructions.text() % localize_user_manual_link(
-                'http://manual.calibre-ebook.com/faq.html#how-do-i-use-calibre-with-my-ipad-iphone-ipod-touch'))
+                'https://manual.calibre-ebook.com/faq.html#how-do-i-use-calibre-with-my-ipad-iphone-ipod-touch'))
         except TypeError:
             pass  # user manual link was already replaced
         self.instructions.setOpenExternalLinks(True)
@@ -750,7 +755,7 @@ class FinishPage(QWizardPage, FinishUI):
         QWizardPage.__init__(self)
         self.setupUi(self)
         try:
-            self.um_label.setText(self.um_label.text() % localize_user_manual_link('http://manual.calibre-ebook.com'))
+            self.um_label.setText(self.um_label.text() % localize_user_manual_link('https://manual.calibre-ebook.com'))
         except TypeError:
             pass  # link already localized
 
